# SPDX-License-Identifier: MPL-2.0
# Copyright (c) 2025 Daniel Schmidt

"""Unit tests for nac_test.pyats_core.ssh.connection_utils module.

This module is intentionally minimal. The connection_utils module primarily
performs string formatting for SSH/Telnet commands, which is covered by
integration tests through the connection manager.

Tests should be added here when error handling or validation logic is
introduced to the module.
"""
