# SPDX-License-Identifier: MPL-2.0
# Copyright (c) 2025 Daniel Schmidt

"""Core reporting module for combined test results across all frameworks."""

from nac_test.core.reporting.combined_generator import CombinedReportGenerator

__all__ = ["CombinedReportGenerator"]
